@@ -4,7 +4,7 @@ library(dplyr)
 library(tidyr)
 
 ## ---- echo = FALSE--------------------------------------------------------------------------------
-options(width = 100)
+old <- options(width = 100)
 w <- weather_dl(station_id = 5401, start =  "2017-01-01", 
                 interval = "month", format = FALSE)
 
@@ -29,3 +29,7 @@ select(n, period, contains("temp_daily"))
 ## ---- echo = FALSE--------------------------------------------------------------------------------
 knitr::kable(codes)
 
+## ---- include = FALSE---------------------------------------------------------
+# Reset options
+options(old)
+
