@@ -1,56 +1,105 @@
 
-# stations_dl ------------------------------------------------------------
-context("stations_dl")
+# stations_dl() ------------------------------------------------------------
+context("stations_dl()")
+
+test_that("stations_dl() requires R 3.3.4", {
+  # Can't test stations_dl because requires depth = 2 which creates problems:
+  # https://github.com/r-lib/mockery/issues
+  stub(stations_dl_internal, 'getRversion', package_version("3.3.3"))
+  expect_message(stations_dl_internal(), "Need R version")
+})
+
+test_that("stations_dl() requires lutz and sf", {
+  stub(stations_dl_internal, 'requireNamespace', mock(FALSE, FALSE))
+  expect_error(stations_dl_internal(), "Package 'lutz' and its dependency, 'sf'")
+})
+
+test_that("stations_dl() errors appropriately", {
+  bkup <- getOption("weathercan.urls.stations")
+
+  options(weathercan.urls.stations = "https://httpstat.us/404")
+  vcr::use_cassette("stations_http404", {
+    expect_error(stations_dl(), "Not Found (HTTP 404).", fixed = TRUE)
+  })
+
+  options(weathercan.urls.stations = bkup)
+})
 
 test_that("stations_normals() gets normals info", {
-  skip_on_cran()
-  skip_on_travis()
-  skip_if_offline()
-  expect_silent(n <- stations_normals(years = "1981-2010")) %>%
-    expect_is("character")
-  expect_gt(length(n), 500)
-  expect_true(all(nchar(n) == 7))
-})
+  vcr::use_cassette("stations_normals", {
+    expect_silent(n <- stations_normals()) %>%
+      expect_is("data.frame")
+  })
 
-test_that("stations_dl() runs and returns data", {
-  skip_on_cran()
-  skip_on_travis()
-  skip_if_offline()
-
-  if(getRversion() < "3.3.4") {
-    expect_message(s <- stations_dl(),
-                   paste0("Need R version 3.3.4 or greater to update the ",
-                          "stations data"))
-  } else if(getRversion() >= "3.3.4") {
-    # If get message about not reachable, try again
-    expect_error({s <- stations_dl()}, regexp = NA)
-    expect_warning(stations_dl(url = "test.csv"))
-    expect_is(s, "data.frame")
-    expect_length(s, 14)
-    expect_lt(length(data.frame(s)[is.na(data.frame(s))]),
-              length(data.frame(s)[!is.na(data.frame(s))]))
-    expect_is(s$prov, "character")
-    expect_is(s$station_name, "character")
-    expect_gt(nrow(s), 10)
-    expect_equal(unique(s$interval), c("day", "hour", "month"))
-
-    # Check content
-    expect_equal(nrow(s[is.na(s$station_name),]), 0)
-    expect_equal(nrow(s[is.na(s$station_id),]), 0)
-    expect_equal(nrow(s[is.na(s$prov),]), 0)
-    expect_true(all(table(s$station_id) == 3)) # One row per time interval type
-  }
+  expect_gt(nrow(n), 1500)
+  expect_named(n, c("station_name", "climate_id",
+                    "normals_1981_2010", "normals_1971_2000"))
 })
 
 
-# stations_search ---------------------------------------------------------
-context("stations_search")
+test_that("stations_dl() runs and updates data", {
+  skip_if_not_installed("sf")
+  skip_if_not_installed("lutz")
+  skip_on_cran()
+  skip_if_offline()
+
+  #vcr::use_cassette("stations_dl_good", {  # Don't use vcr until deal with url redirects
+  #})
+  stub(stations_dl_internal, "Sys.Date", as.Date("2000-01-01"))
+  expect_message(
+    stations_dl_internal(loc = system.file("extdata", package = "weathercan")),
+    "Stations data saved")
+  expect_equal(stations_meta()$weathercan_modified, as.Date("2000-01-01"))
+
+  stub(stations_dl_internal, "Sys.Date", Sys.Date())
+  expect_message(
+    stations_dl_internal(loc = system.file("extdata", package = "weathercan")),
+    "Stations data saved")
+  expect_equal(stations_meta()$weathercan_modified, Sys.Date())
+})
+
+
+# stations() --------------------------------------------------------------
+test_that("stations() /stations_meta() return data",{
+ expect_silent(s <- stations()) %>%
+    expect_s3_class("data.frame")
+
+  expect_silent(stations_meta()) %>%
+    expect_type("list")
+
+  expect_named(stations_meta(), c("ECCC_modified", "weathercan_modified"))
+
+  mockery::stub(stations, "stations_meta",
+                list(weathercan_modified = as.Date("2000-01-01")))
+  expect_message(stations(),
+                 "The stations data frame hasn't been updated in over 4 weeks.")
+
+
+  expect_length(s, 16)
+  expect_lt(length(data.frame(s)[is.na(data.frame(s))]),
+            length(data.frame(s)[!is.na(data.frame(s))]))
+  expect_is(s$prov, "character")
+  expect_is(s$station_name, "character")
+  expect_gt(nrow(s), 10)
+  expect_equal(unique(s$interval), c("day", "hour", "month"))
+
+  # Check content
+  expect_equal(nrow(s[is.na(s$station_name),]), 0)
+  expect_equal(nrow(s[is.na(s$station_id),]), 0)
+  expect_equal(nrow(s[is.na(s$prov),]), 0)
+  expect_true(all(table(s$station_id) == 3)) # One row per time interval type
+
+})
+
+
+# stations_search() ---------------------------------------------------------
+context("stations_search()")
 
 test_that("stations_search 'name' returns correct format", {
   expect_error(stations_search())
   expect_error(stations_search(name = mean()))
   expect_is(stations_search("XXX"), "data.frame")
-  expect_length(stations_search("XXX"), 14)
+  expect_length(stations_search("XXX"), 16)
 
 })
 
@@ -82,11 +131,12 @@ test_that("stations_search 'name' returns correct data", {
 
 
 test_that("stations_search 'coords' returns correct format", {
+  skip_if_not_installed("sf")
   expect_error(stations_search(coords = c("Hi")))
   expect_error(stations_search(coords = 44))
   expect_message(stn <- stations_search(coords = c(54, -122)))
   expect_is(stn, "data.frame")
-  expect_length(stn, 15)
+  expect_length(stn, 17)
   expect_gt(nrow(stn), 0)
 })
 
@@ -179,8 +229,19 @@ test_that("stations_search 'starts_latest' and 'ends_earliest' together", {
 })
 
 test_that("stations_search returns normals only", {
-  expect_silent(s <- stations_search("Brandon", normals_only = TRUE))
-  expect_gt(nrow(stations), nrow(s))
+  expect_warning(s <- stations_search("Brandon", normals_only = TRUE),
+                 "`normals_only` is deprecated")
+  expect_silent(s <- stations_search("Brandon", normals_years = "current"))
+  expect_gt(nrow(stations()), nrow(s))
   expect_true(all(s$normals))
   expect_equal(unique(s$station_id), s$station_id)
+
+  expect_silent(s1 <- stations_search("Brandon", normals_years = "1981-2010"))
+  expect_gt(nrow(stations()), nrow(s1))
+  expect_equal(s$station_id, s1$station_id)
+
+  expect_silent(s2 <- stations_search("Brandon", normals_years = "1971-2000"))
+  expect_gt(nrow(stations()), nrow(s2))
+  expect_equal(s$station_id, s1$station_id)
+
 })
